@@ -4,10 +4,27 @@
 #' functioning procedures on an object
 #' estimated with \code{multipleGroup()}. The compensatory and non-compensatory statistics provided
 #' are described in Chalmers (2018), which generally can be interpreted as IRT generalizations
-#' of the SIBTEST and CSIBTEST statistics. These require the ACOV matrix to be computed in the
+#' of the SIBTEST and CSIBTEST statistics. For hypothesis tests, these measures
+#' require the ACOV matrix to be computed in the
 #' fitted multiple-group model (otherwise, sets of plausible draws from the posterior are explicitly
 #' required).
 #'
+#' The effect sizes estimates by the DRF function are
+#' \deqn{sDRF = \int [S(C|\bm{\Psi}^{(R)},\theta) S(C|\bm{\Psi}^{(F)},\theta)] f(\theta)d\theta,}
+#' \deqn{uDRF = \int |S(C|\bm{\Psi}^{(R)},\theta) S(C|\bm{\Psi}^{(F)},\theta)| f(\theta)d\theta,}
+#' and
+#' \deqn{dDRF = \sqrt{\int [S(C|\bm{\Psi}^{(R)},\theta) S(C|\bm{\Psi}^{(F)},\theta)]^2 f(\theta)d\theta}}
+#' where \eqn{S(.)} are the scoring equations used to evaluate the model-implied
+#' difference between the focal and reference group. The \eqn{f(\theta)}
+#' terms can either be estimated from the posterior via an empirical
+#' histogram approach (default), or can use the best
+#' fitting prior distribution that is obtain post-convergence (default is a Guassian
+#' distribution). Note that, in comparison to Chalmers (2018), the focal group is
+#' the leftmost scoring function while the reference group is the rightmost
+#' scoring function. This is largely to keep consistent with similar effect
+#' size statistics, such as SIBTEST, DFIT, Wainer's measures of impact, etc,
+#' which in general can be seen as special-case estimators of this family.
+#'
 #' @aliases DRF
 #' @param mod a multipleGroup object which estimated only 2 groups
 #' @param draws a number indicating how many draws to take to form a suitable multiple imputation
@@ -42,11 +59,15 @@
 #' @param DIF logical; return a list of item-level imputation properties using the DRF statistics?
 #'   These can generally be used as a DIF detection method and as a graphical display for
 #'   understanding DIF within each item
+#' @param best_fitting logical; use the best fitting parametric distribution (Gaussian by default)
+#'  that was used at the time of model estimation? This will result in much fast computations, however
+#'  the results are more dependent upon the underlying modelling assumptions. Default is FALSE, which
+#'  uses the empirical histogram approach
 #' @param p.adjust string to be passed to the \code{\link{p.adjust}} function to adjust p-values.
 #'   Adjustments are located in the \code{adj_pvals} element in the returned list. Only applicable when
 #'   \code{DIF = TRUE}
 #' @param den.type character specifying how the density of the latent traits is computed.
-#'   Default is \code{'both'} to include the information from both groups,
+#'   Default is \code{'marginal'} to include the proportional information from both groups,
 #'   \code{'focal'} for just the focal group, and \code{'reference'} for the reference group
 #' @param auto.key plotting argument passed to \code{\link{lattice}}
 #' @param par.strip.text plotting argument passed to \code{\link{lattice}}
@@ -88,11 +109,18 @@
 #' plot(mod, type = 'itemscore')
 #' plot(mod, type = 'itemscore', which.items = 10:15)
 #'
+#' # empirical histogram approach
 #' DRF(mod)
 #' DRF(mod, focal_items = 6:10) #DBF
 #' DRF(mod, DIF=TRUE)
 #' DRF(mod, DIF=TRUE, focal_items = 10:15)
 #'
+#' # Best-fitting Gaussian distributions
+#' DRF(mod, best_fitting=TRUE)
+#' DRF(mod, focal_items = 6:10, best_fitting=TRUE) #DBF
+#' DRF(mod, DIF=TRUE, best_fitting=TRUE)
+#' DRF(mod, DIF=TRUE, focal_items = 10:15, best_fitting=TRUE)
+#'
 #' DRF(mod, plot = TRUE)
 #' DRF(mod, focal_items = 6:10, plot = TRUE) #DBF
 #' DRF(mod, DIF=TRUE, plot = TRUE)
@@ -100,9 +128,11 @@
 #'
 #' mirtCluster()
 #' DRF(mod, draws = 500)
+#' DRF(mod, draws = 500, best_fitting=TRUE)
 #' DRF(mod, draws = 500, plot=TRUE)
 #'
 #' # pre-draw parameter set to save computations
+#' #  (more useful when using non-parametric bootstrap)
 #' param_set <- draw_parameters(mod, draws = 500)
 #' DRF(mod, focal_items = 6, param_set=param_set) #DIF
 #' DRF(mod, DIF=TRUE, param_set=param_set) #DIF
@@ -257,7 +287,8 @@
 #
 #' }
 DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
-                param_set = NULL, den.type = 'both', CI = .95, npts = 1000,
+                param_set = NULL, den.type = 'marginal', best_fitting=FALSE,
+                CI = .95, npts = 1000,
                 quadpts = NULL, theta_lim=c(-6,6), Theta_nodes = NULL,
                 plot = FALSE, DIF = FALSE, p.adjust = 'none',
                 par.strip.text = list(cex = 0.7),
@@ -296,7 +327,7 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
         ret
     }
 
-    fn <- function(x, omod, Theta, max_score, Theta_nodes = NULL,
+    fn <- function(x, omod, Theta, max_score, Theta_nodes = NULL, best_fitting,
                    plot, DIF, focal_items, details, signs=NULL, rs=NULL, den.type){
         mod <- omod
         if(!is.null(Theta_nodes)){
@@ -304,12 +335,12 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
                                 which.items=focal_items)
             T2 <- expected.test(mod, Theta_nodes, group=2L, mins=FALSE, individual=DIF,
                                 which.items=focal_items)
-            ret <- T1 - T2
+            ret <- T2 - T1
             if(!DIF) ret <- c("sDRF." = ret)
             return(ret)
         }
-        calc_DRFs(mod=mod, Theta=Theta, plot=plot, max_score=max_score, DIF=DIF,  den.type=den.type,
-                  focal_items=focal_items, details=details, signs=signs, rs=rs)
+        calc_DRFs(mod=mod, Theta=Theta, plot=plot, max_score=max_score, DIF=DIF, den.type=den.type,
+                  focal_items=focal_items, details=details, signs=signs, rs=rs, best_fitting=best_fitting)
     }
     fn2 <- function(ind, pars, MGmod, param_set, rslist, ...){
         pars <- reloadPars(longpars=param_set[ind,],
@@ -321,12 +352,12 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
     }
 
     if(missing(mod)) missingMsg('mod')
-    stopifnot(den.type %in% c('both', 'focal', 'reference'))
+    stopifnot(den.type %in% c('marginal', 'focal', 'reference'))
     stopifnot(is.logical(plot))
     if(DIF && !is.null(Theta_nodes))
         stop('DIF must be FALSE when using Theta_nodes', call.=FALSE)
     type <- 'score'
-    if(class(mod) != 'MultipleGroupClass')
+    if(!is(mod, 'MultipleGroupClass'))
         stop('mod input was not estimated by multipleGroup()', call.=FALSE)
     if(mod@Data$ngroups != 2L)
         stop('DTF only supports two group models at a time', call.=FALSE)
@@ -389,7 +420,7 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
                     technical = list(storeEtable=TRUE, theta_lim=theta_lim, omp=FALSE),
                     quadpts=quadpts, large=large, TOL = NaN)
     if(plot) Theta_nodes <- matrix(seq(theta_lim[1L], theta_lim[2L], length.out=1000))
-    oCM <- lapply(1L, fn, omod=mod, Theta_nodes=Theta_nodes,
+    oCM <- lapply(1L, fn, omod=mod, Theta_nodes=Theta_nodes, best_fitting=best_fitting,
                   max_score=max_score, Theta=Theta, plot=plot, den.type=den.type,
                   DIF=DIF, focal_items=focal_items, details=details)[[1L]]
     signs <- attr(oCM, 'signs')
@@ -408,7 +439,7 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
             reloadPars(longpars=longpars, pars=pars, ngroups=2L, J=length(pars[[1L]])-1L)
         })
         list_scores <- myLapply(1L:nrow(param_set), fn2, progress=verbose,
-                                pars=pars, MGmod=mod, param_set=param_set,
+                                pars=pars, MGmod=mod, param_set=param_set, best_fitting=best_fitting,
                                 max_score=max_score, Theta=Theta, rslist=rslist,
                                 Theta_nodes=Theta_nodes, plot=plot, details=details,
                                 DIF=DIF, focal_items=focal_items, signs=signs, den.type=den.type)
@@ -418,7 +449,8 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
         if(plot) return(plot.DRF(Theta_nodes, oCM, CIs=scores, DIF=DIF, CI=CI,
                                  itemnames = extract.mirt(mod, 'itemnames')[focal_items], ...))
         CIs <- apply(scores, 2, bs_range, CI=CI)
-        CIs <- CIs[,1L:(ncol(CIs)/2L)]
+        if(is.null(Theta_nodes))
+            CIs <- CIs[,1L:(ncol(CIs) * 3/5)]
         rownames(CIs) <- c(paste0('CI_', round((1-CI)/2, 3L)*100),
                            paste0('CI_', round(CI + (1-CI)/2, 3L)*100))
         if(!is.null(Theta_nodes))
@@ -426,14 +458,18 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
         if(DIF){
             oCM <- matrix(oCM, length(focal_items))
             t1 <- compute_ps(oCM[,1L], scores[,1L:length(focal_items), drop=FALSE])
-            t2 <- compute_ps(oCM[,3L:4L], scores[,1L:(length(focal_items)*2L) + length(focal_items)*2L, drop=FALSE],
+            t2 <- compute_ps(oCM[,3L:4L], scores[,1L:(length(focal_items)*2L) +
+                                                     length(focal_items)*2L, drop=FALSE],
                              X2=TRUE)
             ret <- list(sDIF = as.mirt_df(data.frame(sDIF = oCM[,1L],
                                           t(CIs[,1L:length(focal_items)]),
                                           t1, row.names = focal_items)),
                         uDIF = as.mirt_df(data.frame(uDIF = oCM[,2L],
                                           t(CIs[,1L:length(focal_items) + length(focal_items)]),
-                                          t2, row.names=focal_items)))
+                                          t2, row.names=focal_items)),
+                        dDIF = as.mirt_df(data.frame(dDIF = oCM[,3L],
+                                                     t(CIs[,1L:length(focal_items) + length(focal_items)*2]),
+                                                     row.names=focal_items)))
             if(p.adjust != 'none'){
                 ret$sDIF$adj_pvals <- p.adjust(ret$sDIF$p, method=p.adjust)
                 ret$uDIF$adj_pvals <- p.adjust(ret$uDIF$p, method=p.adjust)
@@ -441,20 +477,21 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
         } else {
             t1 <- compute_ps(oCM[1L], scores[,1L])
             t2 <- compute_ps(oCM[3L:4L], scores[,3L:4L], X2=TRUE)
-            tests <- rbind(t1, t2)
+            tests <- rbind(t1, t2, NA)
             ret <- data.frame(n_focal_items=length(focal_items),
-                              stat = oCM[1L:2L], t(CIs), tests, check.names = FALSE)
+                              stat = oCM[1L:3L], t(CIs), tests, check.names = FALSE)
             ret <- as.mirt_df(ret)
         }
     } else {
         # no imputations
         if(DIF){
-            ret <- data.frame(matrix(oCM, length(oCM)/4L), row.names = focal_items)
-            ret <- ret[,-c(3L:4L)]
-            colnames(ret) <- c('sDIF', 'uDIF')
+            ret <- data.frame(matrix(oCM, length(oCM)/5L), row.names = focal_items)
+            ret <- ret[,-c(4L:5L)]
+            colnames(ret) <- c('sDIF', 'uDIF', 'dDIF')
             ret <- as.mirt_df(ret)
         } else {
-            ret <- data.frame(n_focal_items=length(focal_items), sDRF=oCM[1L], uDRF=oCM[2L],
+            ret <- data.frame(n_focal_items=length(focal_items),
+                              sDRF=oCM[1L], uDRF=oCM[2L], dDRF=oCM[3L],
                               row.names=NULL)
             ret <- as.mirt_df(ret)
         }
@@ -463,7 +500,7 @@ DRF <- function(mod, draws = NULL, focal_items = 1L:extract.mirt(mod, 'nitems'),
 }
 
 calc_DRFs <- function(mod, Theta, DIF, plot, max_score, focal_items, details, den.type,
-                      rs=NULL, signs=NULL){
+                      best_fitting, rs=NULL, signs=NULL){
     if(DIF){
         T1 <- expected.test(mod, Theta, group=1L, mins=FALSE, individual = TRUE,
                             which.items=focal_items)
@@ -476,24 +513,42 @@ calc_DRFs <- function(mod, Theta, DIF, plot, max_score, focal_items, details, de
                             mins=FALSE, which.items=focal_items))
     }
     if(plot) return(c(T1, T2))
-    if(is.null(rs)){
-        mod2 <- with(details, multipleGroup(data=data, model=model, group=group, itemtype=itemtype, large=large,
-                                            constrain=constrain, quadpts=quadpts, TOL=TOL,
-                                            pars=mod2values(mod), technical=technical))
-        r1 <- rowSums(mod2@Internals$Etable[[1L]]$r1)
-        r2 <- rowSums(mod2@Internals$Etable[[2L]]$r1)
+    if(best_fitting){
+        pars <- extract.mirt(mod, 'pars')
+        nitems <- extract.mirt(mod, 'nitems')
+        den1 <- pars[[1]]@ParObjects$pars[[nitems + 1L]]@den(
+            pars[[1]]@ParObjects$pars[[nitems + 1L]], Theta)
+        den1 <- den1 / sum(den1)
+        den2 <- pars[[2]]@ParObjects$pars[[nitems + 1L]]@den(
+            pars[[2]]@ParObjects$pars[[nitems + 1L]], Theta)
+        den2 <- den2 / sum(den2)
+        p <- if(den.type == 'marginal'){
+            Ns <- table(extract.mirt(mod, 'group'))
+            (Ns[1] * den1 + Ns[2] * den2) / sum(Ns)
+        } else if(den.type == 'focal') den2
+          else den1
     } else {
-        r1 <- rs[,1L]
-        r2 <- rs[,2L]
+        if(is.null(rs)){
+            mod2 <- with(details, multipleGroup(data=data, model=model, group=group,
+                                                itemtype=itemtype, large=large,
+                                                constrain=constrain, quadpts=quadpts, TOL=TOL,
+                                                pars=mod2values(mod), technical=technical))
+            r1 <- rowSums(mod2@Internals$Etable[[1L]]$r1)
+            r2 <- rowSums(mod2@Internals$Etable[[2L]]$r1)
+        } else {
+            r1 <- rs[,1L]
+            r2 <- rs[,2L]
+        }
+        p <- if(den.type == 'marginal')
+            (r1 + r2) / sum(r1 + r2)
+        else if(den.type == 'focal')
+            r2 / sum(r2)
+        else r1/ sum(r1)
     }
-    p <- if(den.type == 'both')
-        (r1 + r2) / sum(r1 + r2)
-    else if(den.type == 'focal')
-        r2 / sum(r2)
-    else r1/ sum(r1)
-    D <- T1 - T2
+    D <- T2 - T1
     uDRF <- colSums(abs(D) * p)
     sDRF <- colSums(D * p)
+    dDRF <- sqrt(colSums(D^2 * p))
     attach_signs <- FALSE
     ret <- if(is.null(signs)){
         signs <- D < 0
@@ -501,7 +556,8 @@ calc_DRFs <- function(mod, Theta, DIF, plot, max_score, focal_items, details, de
     }
     uDRF_L <- colSums(D * p * signs)
     uDRF_U <- colSums(D * p * !signs)
-    ret <- c(sDRF=sDRF, uDRF=uDRF, uDRF_L=uDRF_L, uDRF_U=uDRF_U)
+    ret <- c(sDRF=sDRF, uDRF=uDRF, dDRF=dDRF,
+             uDRF_L=uDRF_L, uDRF_U=uDRF_U)
     if(attach_signs) attr(ret, 'signs') <- signs
     ret
 }
@@ -509,7 +565,7 @@ calc_DRFs <- function(mod, Theta, DIF, plot, max_score, focal_items, details, de
 #' Draw plausible parameter instantiations from a given model
 #'
 #' Draws plausible parameters from a model using parametric sampling (if the information matrix
-#' was computed) or via boostrap sampling. Primarily for use with the \code{\link{DRF}} function.
+#' was computed) or via bootstrap sampling. Primarily for use with the \code{\link{DRF}} function.
 #'
 #' @param mod estimated single or multiple-group model
 #' @param draws number of draws to obtain
@@ -608,6 +664,7 @@ draw_parameters <- function(mod, draws, method = c('parametric', 'boostrap'),
                         shortpars=shortpars, longpars=longpars, lbound=lbound,
                         ubound=ubound, pre.ev=pre.ev, constrain=constrain, est=est,
                         imputenums=imputenums, MGmod=mod, redraws=redraws, pars=pars)
+        if(verbose) cat("\n")
         ret <- do.call(rbind, ret)
         if(any(logits))
             ret[,logits] <- antilogit(ret[,logits])
